#!/bin/env python
"""Generate sut java implementation from uppaal model"""
# Author: Harco Kuppens


import sys, os,  pprint, re, getpass, tempfile, subprocess, shutil  # standard libraries
import argparse # only in standard lib for python 2.7 and later -> installed local copy  (pure python versions)
import yaml  # external libraries   -> installed local copy   (pure python versions) (not yaml doesn't use libyaml but pure python version)
import pyuppaal  # local libraries          -> installed local copy   (pure python versions)



import model_parser
import gentemplate
import util

def check_dependencies():
    util.check_program('java','.exe')



def uppaal2lotos(model_file,destdir,range_min,range_max):
    """
    destdir : create on the fly; the enduser should himself check first if this is possible
    """

    # resource dir is relative from this module :
    resource_dir = os.path.abspath( os.path.dirname(os.path.abspath(__file__)) + '/../resources/' )
    # cp lotos stuff in destdir
    srcdir=os.path.join(resource_dir, 'lotos')
    shutil.copytree(srcdir,destdir)


    # put a copy of model in output dir
    new_model_file=os.path.join(destdir, 'model.orig.xml')
    shutil.copy(model_file, new_model_file)


    uppaal2lotos_jar = os.path.abspath( os.path.dirname(os.path.abspath(__file__)) + '/../java/Uppaal2Lotos.jar' )

    # generate lotos model
    #command=['java', '-jar', uppaal2lotos_jar , new_model_file]
    command=['java', '-jar', uppaal2lotos_jar , new_model_file, range_min, range_max ]
    output=util.run_command_with_output(command,input=None,cwd=destdir)

    lotos_file=os.path.join( destdir , 'model.lotos')
    f=open(lotos_file,'w')
    f.write(output)
    f.close()

    # add settings for lotos in model.t file
    out_file=os.path.join( destdir , 'model.t')
    template_data={
     'lts_min_value': range_min,
     'lts_max_value': range_max
    }
    templates_dir = os.path.abspath( os.path.dirname(os.path.abspath(__file__)) + '/templates/' )
    gentemplate.fill_template(out_file,template_data,templates_dir,'model.t')



#-------------------------------------------------------------------------------
#   main + input parameters parsing
#-------------------------------------------------------------------------------


def args_parsing(args):

    argparser = argparse.ArgumentParser(prog=os.path.basename(args[0]))
    argparser.add_argument('modelfile', help='input uppaal model file')
    argparser.add_argument('outputdir', help='output directory where to store lotos file and related files')
    argparser.add_argument('range_min', help='minimum value of range used for input params when flattening the model')
    argparser.add_argument('range_max', help='maximum value of range used for input params when flattening the model')

    str=argparser.format_help()
    outputdir_notice_str= "note: 'outputdir' must be a none existing directory and will be created on the fly"
    def myhelp( ):
        print str
        print "\n" + outputdir_notice_str


    def myerror( message):
            print >>sys.stderr, ('%s: error: %s\n') % (argparser.prog, message)
            argparser.print_usage(sys.stderr)

            print "\n" + outputdir_notice_str
            exit(1)

    argparser.error=myerror
    argparser.format_help=myhelp

    parse_result=argparser.parse_args(args[1:])

    # handle modelfile argument
    model_file=parse_result.modelfile
    model_file=os.path.abspath(model_file)
    # always use unix slashes in python!!
    # automatic convert any windows slashes
    # TODO: document this convention in python/txtdocs/
    model_file.replace('\\','/')

    util.check_isreadablefile(model_file)

    # handle output directory argument
    # - first get absolute path
    output_dir=parse_result.outputdir
    output_dir=os.path.abspath(output_dir)   # on windows also interprets / as dir sep, and returns path with windows \
    output_dir.replace('\\','/')
    # - check if we can create the output directory on the fly
    if os.path.isdir(output_dir):
        util.app_error("output directory '" + output_dir + "' already exists")
    parent_dir=util.parentdir(output_dir)
    if not util.iswritabledir(parent_dir):
        util.app_error("output directory '" + output_dir + "' cannot be created because parent directory is not writable")


    range_min=parse_result.range_min
    range_max=parse_result.range_max

    return  [model_file,output_dir,range_min,range_max]




def main(args):
    check_dependencies()

    # parse arguments
    (model_file,output_dir,range_min,range_max)=args_parsing(args)

    # convert uppaal model 2 lotos
    uppaal2lotos(model_file,output_dir,range_min,range_max)



if __name__ == "__main__":
    main(sys.argv)


